/* ======================================================================
 * 
 *  CCCCC          drawing.c
 * CC   CC         Author(s): Bill Sherman
 * CC              Created: (unknown, sometime before June 20, 1998)
 * CC   CC         Last Modified: June 5, 2003
 *  CCCCC
 * 
 * Code file for a sample VR application to render a simple world.
 *   A lot of this code came from my VR programming training examples.
 * 
 * Copyright 2003, Bill Sherman & Friends, All rights reserved.
 * With the intent to provide an open-source license to be named later.
 * ====================================================================== */
#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#if defined(FREEVR)
#  include "freevr.h"
#else
   void vrPrintf() { }	/* needed for vr_basicgfx.glx.h */
#  define	vrDbgDo(n)	1
#  define	vrPrintf	printf
#  define	vrErrPrintf	printf
#  define	vrFprintf	fprintf
#endif

#include "vr_basicgfx.glx.h"

#define	LIMITED_TRACE


/*********************************************************************/
/* init_gfx(): initialize general graphics properties (eg. lighting) */
/*********************************************************************/
void init_gfx(vrRenderInfo *rendinfo)
{
static	int	num_init = 0;

#ifdef UNPICKY_COMPILER
	GLfloat	light_position[] = { 1.0, 1.0, 9.0, 1.0 }; 
	GLfloat	light_diffuse[] = { 0.8, 0.8, 0.8, 1.0 }; 
	GLfloat	light_ambient[] = { 0.3, 0.3, 0.3, 1.0 }; 
#endif

#if defined(FREEVR)
	vrPrintf("App: " BOLD_TEXT "Entering init_gfx().  When am I?\n" NORM_TEXT);
	vrPrintf("App: init graphics for context %p, persp %p, window %p (%d), eye %p\n",
		rendinfo->context, rendinfo->persp, rendinfo->window, rendinfo->window->num, rendinfo->eye);
#endif
	/* If we init too many times, there is probably a bug, so abort and dump core */
	num_init++;
	if (num_init == 10) abort();


	glShadeModel(GL_SMOOTH);
	glEnable(GL_LINE_SMOOTH);
	glLineWidth(2.0);

	glDrawBuffer(GL_FRONT_AND_BACK);
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClearDepth(1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDrawBuffer(GL_BACK);

#if 0
	/* TODO: when using a lighting model, objects without a */
	/*   material will appear as shades of grey.            */
	glLightfv(GL_LIGHT0, GL_POSITION, light_position); 
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse); 
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient); 
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
#endif
}


/*********************************************************************/
/* draw_world(): draw a simple world.                                */
/*********************************************************************/
void draw_world(vrRenderInfo *rendinfo)
{
	char		buffer[2048];	/* this could be smaller, but is now safer against      */
					/*   memory overflow bugs.  TODO: replace sprintf's     */
					/*   with snprintf when that becomes ubiquitous -- it's */
					/*   not in IRIX 6.2                                    */
	vrInputInfo	*inputinfo;	/* Pointer to the overall input data structure */

#if defined(FREEVR) /* { */
	vrTime		beginRenderTime = vrCurrentWallTime();/* Time when we begin the rendering process */
	vrTime		renderTime;	/* amount of time spent rendering */

#endif /* } */

#if 0
int i;
for (i = 0; i < 200; i++);
vrSleep(3);
#endif

#if 0
	int		val;
	do {
		val = glGetError();
		vrPrintf(RED_TEXT "glGetError returned %d\n" NORM_TEXT, val);
	} while (val != 0);
#endif

#if defined(FREEVR) && 0
	vrPrintf("App: render graphics for context %p, persp %p, window %p (%d), eye %p (%d)\n",
		rendinfo->context, rendinfo->persp,
		rendinfo->window, rendinfo->window->num,
		rendinfo->eye, rendinfo->eye->num);
#endif
	vrTrace("draw_world", "beginning");

	inputinfo = rendinfo->context->input;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* draw colored axes at the origin: red=x, green=y, blue=z */
	glBegin(GL_LINES);
		glColor3ub(128, 0, 0);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(2.0, 0.0, 0.0);

		glColor3ub(0, 128, 0);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(0.0, 2.0, 0.0);

		glColor3ub(0, 0, 128);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(0.0, 0.0, 2.0);
	glEnd();

#if defined(FREEVR) /* { */
	/* draw colored axes at the head: red=x, green=y, blue=z */
	if (rendinfo->window->mount == VRWINDOW_SIMULATOR) {
		glPushMatrix();
		vrRenderTransform6sensor(0);
		glBegin(GL_LINES);
			glColor3ub(128, 0, 0);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(2.0, 0.0, 0.0);

			glColor3ub(0, 128, 0);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(0.0, 2.0, 0.0);

			glColor3ub(0, 0, 128);
			glVertex3f(0.0, 0.0, 0.0);
			glVertex3f(0.0, 0.0, 2.0);
		glEnd();
		glPopMatrix();
	}
#endif /* } */


#if 0 /* a simple two object grey world for red-green testing */

	/* NOTE: start in RW coordinate space */

	/* a 2' 50% grey cube just above the floor of the cave */
	glPushMatrix();
	glColor3ub(128, 128, 128);
	glTranslatef(-2.5, 2.0, -0.5);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	glPushMatrix();
#if defined(FREEVR) /* { */
	vrRenderTransformUserTravel();	/* now in VW coordinate space */
#endif /* } */

	/* an 80% grey pyramid near eye-height */
	glPushMatrix();
	glColor3ub(204, 204, 204);
	glTranslatef(-2.3, 6.2, -0.5);
	glScalef(1.5, 1.5, 1.5);
	glRotatef(90.0, 1.0, 0.0, 0.0);
#if 0
	vrShapeGLPyramidPlusOutline();
#else
	vrShapeGLPyramid();
#endif
	glPopMatrix();

	glPopMatrix();			/* back to RW coordinate space */


#elif 0 /* a different simple test world for making a diagram */

	/* NOTE: start in RW coordinate space */

	/* a 2' 50% grey cube near the center of the cave */
	glPushMatrix();
	glColor3ub(128, 128, 128);
	glTranslatef( 0.5, 4.2, -2.5);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	glPushMatrix();
#if defined(FREEVR) /* { */
	vrRenderTransformUserTravel();	/* now in VW coordinate space */
#endif /* } */

	/* an 80% grey pyramid near eye-height */
	glPushMatrix();
	glColor3ub(204, 204, 204);
	glTranslatef( 0.5, 6.0, -6.5);
	glScalef(1.5, 1.5, 1.5);
	vrShapeGLPyramidPlusOutline();
	glPopMatrix();

	glPopMatrix();			/* back to RW coordinate space */

#elif 1 /* a somewhat more complicated test world */

#ifndef LIMITED_TRACE
	vrTrace("draw_world", "the more complicated test world");
#endif
	/* NOTE: start in RW coordinate space */

	/* a 1' rose cube behind the cave */
	glPushMatrix();
	glColor3ub(240, 138, 133);
	glTranslatef(0.0, 5.0, 7.0);
	glScalef(0.5, 0.5, 0.5);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 4' blue cube in front of the cave */
	glPushMatrix();
	glColor3ub(0, 0, 255);
	glTranslatef(0.0, 3.0, -7.0);
	glScalef(2.0, 2.0, 2.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 4' white cube left of the cave */
	glPushMatrix();
	glColor3ub(255, 255, 255);
	glTranslatef(-7.0, 3.0, 0.0);
	glScalef(2.0, 2.0, 2.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a pink pyramid on the floor of the cave near the front wall */
	glPushMatrix();
	glColor3ub(255, 100, 100);
	glTranslatef(0.0, 1.0, -4.0);
	vrShapeGLPyramidPlusOutline();
	glPopMatrix();

	/* a 2' light purple cube at the front left corner of the cave */
	glPushMatrix();
	glColor3ub(100, 100, 255);
	glTranslatef(-6.0, 7.0, -6.0);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 2' green cube below the cave */
	glPushMatrix();
	glColor3ub(0, 255, 0);
	glTranslatef(5.0, 1.0, 0.0);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 2' dark purple cube below the center of the cave */
	glPushMatrix();
	glColor3ub(100, 0, 255);
	glTranslatef(0.0, -1.0, 0.0);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 10' tall red pillar inside the right front corner of the cave */
	glPushMatrix();
	glColor3ub(240, 50, 50);
	glTranslatef(4.5, 5.0, -4.5);
	glScalef(0.5, 5.0, 0.5);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 10' deep cyan pillar inside the right front corner of the cave */
	glPushMatrix();
	glColor3ub(50, 240, 240);
	glTranslatef(4.5, -5.0, -4.5);
	glScalef(0.5, 5.0, 0.5);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 4' tall yellow pillar near the right wall of the cave */
	glPushMatrix();
	glColor3ub(240, 240, 50);
	glTranslatef(-2.5, 2.0, 2.0);
	glScalef(0.5, 2.0, 0.5);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

#  if 0 /* temporarily turned off so we can see the front screen */
	/* a 4' long orange pillar out of the right wall of the cave */
	glPushMatrix();
	glColor3ub(240, 130, 30);
	glTranslatef(-3.0, 2.0, -2.5);
	glScalef(2.0, 0.5, 0.5);
	vrShapeGLCubePlusOutline();
	glPopMatrix();
#  endif

	glPushMatrix();
#if defined(FREEVR) /* { */
	vrRenderTransformUserTravel();	/* now in VW coordinate space */
#endif /* } */

	/* an 80% grey pyramid near eye-height */
	glPushMatrix();
	glColor3ub(204, 204, 204);
	glTranslatef(-2.3, 6.2, -0.5);
	glScalef(1.5, 1.5, 1.5);
	glRotatef(90.0, 1.0, 0.0, 0.0);
	vrShapeGLPyramidPlusOutline();
	glPopMatrix();

	glPopMatrix();			/* back to RW coordinate space */

#ifndef LIMITED_TRACE
	vrTrace("draw_world", "end of the more complicated test world");
#endif

#else /* another somewhat complex test world */

	/* NOTE: start in RW coordinate space */

	/* a 6' blue cube left of the cave */
	glPushMatrix();
	glColor3ub(0, 0, 255);
	glTranslatef(-10.0, 3.0, 0.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 6' dark purple cube above the user */
	glPushMatrix();
	glColor3ub(100, 0, 255);
	glTranslatef(0.0, 10.0, 0.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 6' red cube right of the cave */
	glPushMatrix();
	glColor3ub(255, 0, 0);
	glTranslatef(10.0, 3.0, 0.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 6' green cube below the cave */
	glPushMatrix();
	glColor3ub(0, 255, 0);
	glTranslatef(0.0, -10.0, 0.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 6' light purple cube behind the cave */
	glPushMatrix();
	glColor3ub(100, 100, 255);
	glTranslatef(0.0, 3.0, 10.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a 6' white cube in front of the cave */
	glPushMatrix();
	glColor3ub(255, 255, 255);
	glTranslatef(0.0, 3.0, -10.0);
	glScalef(3.0, 3.0, 3.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	/* a pyramid sticking through the floor of the cave near the right wall */
	glPushMatrix();
	glColor3ub(255, 100, 100);
	glTranslatef(4.0, 0.0, 0.0);
	vrShapeGLPyramidPlusOutline();
	glPopMatrix();
#endif

#if 0
	glColor3ub(100,255,100);
	vrShapeGLFloor();
#endif

#if defined(FREEVR) /* { */
	/**************************/
	/** Print some text info **/
	/**************************/

#if 1 /* Give detailed Text info: { */
  {
#include "vr_math.h"
#include "vr_procs.h"
	vrMatrix	head;
	vrProcessInfo	*input_proc;
	vrPoint		headloc;
	vrPoint		headlocvw;
	vrPoint		proploc;
	vrVector	vec;
	vrPoint		drawto;
	int		count;

#ifndef LIMITED_TRACE
	vrTrace("draw_world", "doing the text rendering");
#endif

	if (vrMatrixGetRWFromUserHead(&head, 0) != NULL) {
		vrPointGetRWLocationFromMatrix(&headloc, &head);
		vrPointGetVWFromUserMatrix(&headlocvw, 0, &head);

		/* first store, then adjust some graphics attributes for vector rendering */
		glPushAttrib(GL_COLOR_BUFFER_BIT | GL_LINE_BIT);
		glLineWidth(5.0);
		glDisable(GL_BLEND);

	    /* only draw these lines in simulator views */
	    if (rendinfo->window->mount == VRWINDOW_SIMULATOR) {
		/* draw a red line connecting the user to the back-low-center of the CAVE */
		drawto.v[VR_X] = 0.0;
		drawto.v[VR_Y] = 0.0;
		drawto.v[VR_Z] = 5.0;
		glColor4ub(255, 0, 0, 255);
		glBegin(GL_LINE_STRIP);
			glVertex3dv(headloc.v);
			glVertex3dv(drawto.v);
		glEnd();

		/* draw a green line pointing out the front of the user */
		vrVectorGetRWFrom6sensorDir(&vec, 0, VRVECTOR_FORE);
		vrPointAddScaledVector(&drawto, &headloc, &vec, 2.0);
		glColor4ub(0, 255, 0, 255);
		glBegin(GL_LINE_STRIP);
			glVertex3dv(headloc.v);
			glVertex3dv(drawto.v);
		glEnd();

		/* draw a blue line pointing from the user to center of grey pyramid */
		drawto.v[VR_X] = -2.3;
		drawto.v[VR_Y] =  6.2;
		drawto.v[VR_Z] = -0.5;
		vrPointGetRWFromVWUserPoint(&drawto, 0, &drawto);
		glColor4ub(0, 0, 255, 255);
		glBegin(GL_LINE_STRIP);
			glVertex3dv(headloc.v);
			glVertex3dv(drawto.v);
		glEnd();

	glPushMatrix();
	vrRenderTransformUserTravel();	/* now in VW coordinate space */
		/* draw a magenta line pointing out of the user toward the front of the VW */
		/* NOTE: this should be the same as the green line coming from the user's  */
		/*   nose, but in this case we're doing all the calculations in VW space.  */
		vrVectorGetVWFromUser6sensorDir(&vec, 0, 0, VRVECTOR_FORE);
		vrPointAddScaledVector(&drawto, &headlocvw, &vec, 1.8);
		glColor4ub(255, 0, 255, 255);
		glLineWidth(8.0);
		glBegin(GL_LINE_STRIP);
			glVertex3dv(headlocvw.v);
			glVertex3dv(drawto.v);
		glEnd();
		glLineWidth(5.0);
	glPopMatrix();
	    }

		/* draw a yellow line pointing out the front of the "wand" */
		vrPointGetRWFrom6sensor(&proploc, 1);
		vrVectorGetRWFrom6sensorDir(&vec, 1, VRVECTOR_FORE);
		vrPointAddScaledVector(&drawto, &proploc, &vec, 2.0);
		glColor4ub(255, 255, 0, 255);
		glBegin(GL_LINE_STRIP);
			glVertex3dv(proploc.v);
			glVertex3dv(drawto.v);
		glEnd();

		glPopAttrib();

#if 0
		sprintf(buffer, "drawto: (%6.2f %6.2f %6.2f)", drawto.v[VR_X], drawto.v[VR_Y], drawto.v[VR_Z]);
		glRasterPos3f(-6.5,  13.0, -5.01);
		vrRenderText(buffer);
#endif
	}


	/* first store, then adjust some graphics attributes for text rendering */
	/*      disregard z-buffer -- always display the text on top */
	glPushAttrib(GL_DEPTH_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);

	/* set the text color */
	glColor3ub(155, 155, 155);


	/* render on left wall */
	if (inputinfo->num_6sensors >= 1) {
		input_proc = inputinfo->sensor6[0]->my_device->proc;
#if 0
		sprintf(buffer, "input frame: %11d", input_proc->frame_count);
		glRasterPos3f(-5.0,  8.5, 1.5);
		vrRenderText(buffer);

		sprintf(buffer, "input rates: %5.2f(1) %5.2f(10)", input_proc->fps1, input_proc->fps10);
		glRasterPos3f(-5.0,  8.0, 1.5);
		vrRenderText(buffer);

		sprintf(buffer, "render frame: %10d", rendinfo->frame_count);
		glRasterPos3f(-5.0,  7.5, 1.5);
		vrRenderText(buffer);

		sprintf(buffer, "render rates: %5.2f(1) %5.2f(10)", vrThisProc->fps1, vrThisProc->fps10);
		glRasterPos3f(-5.0,  7.0, 1.5);
		vrRenderText(buffer);
#else
		sprintf(buffer, "Shared memory usage:%7ld (%ld freed)", vrShmemUsage(), vrShmemFreed());
		glRasterPos3f(-5.0,  8.0, 1.75);
		vrRenderText(buffer);

		sprintf(buffer, "input info:%9ld %6.2f(1) %6.2f(10)",
			input_proc->frame_count, input_proc->fps1, input_proc->fps10);
		glRasterPos3f(-5.0,  7.5, 1.75);
		vrRenderText(buffer);

		sprintf(buffer, "render info:%8ld %6.2f(1) %6.2f(10)",
			rendinfo->frame_count, vrThisProc->fps1, vrThisProc->fps10);
		glRasterPos3f(-5.0,  7.0, 1.75);
		vrRenderText(buffer);
#endif
	}

	/* render on front wall */
#if 0
	sprintf(buffer, "$YO = '%s' sim time: %8.4f", getenv("YO"), (double)(rendinfo->frame_stime));
#else
	sprintf(buffer, "sim time: %8.4f", (double)(rendinfo->frame_stime));
#endif
	glRasterPos3f(-4.5,  9.0, -5.01);
	vrRenderText(buffer);

	glRasterPos3f(-4.5,  7.0, -5.01);
	sprintf(buffer, "buttons:");
	for (count = 0; count < inputinfo->num_2ways; count++) 
		sprintf(buffer, "%s %d", buffer, vrGet2switchValue(count));
	vrRenderText(buffer);

	if (inputinfo->num_valuators > 0) {
		sprintf(buffer, "valuators:");
		for (count = 0; count < inputinfo->num_valuators; count++) 
			sprintf(buffer, "%s %6.3f", buffer, vrGetValuatorValue(count));
		glRasterPos3f(-4.5,  6.0, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_Nways > 0) {
		glRasterPos3f(-4.5,  6.5, -5.01);
		sprintf(buffer, "nswitches:");
		for (count = 0; count < inputinfo->num_Nways; count++) 
			sprintf(buffer, "%s %d", buffer, vrGetNswitchValue(count));
		vrRenderText(buffer);
	}

	if (vrMatrixGetRWFromUserHead(&head, 0) != NULL) {
		vrPoint	*leye = &(vrCurrentEye()->loc);

		sprintf(buffer, "head: %5.2f %5.2f %5.2f", VRMAT_ROWCOL(&head, VR_X, VR_W), VRMAT_ROWCOL(&head, VR_Y, VR_W), VRMAT_ROWCOL(&head, VR_Z, VR_W));
		glRasterPos3f(-5.0,  5.0,  4.50);
		vrRenderText(buffer);
		glRasterPos3f(-4.5,  5.0, -5.01);	/* ditto this to left wall */
		vrRenderText(buffer);

		sprintf(buffer, "leye: %5.2f %5.2f %5.2f", leye->v[VR_X], leye->v[VR_Y], leye->v[VR_Z]);
		glRasterPos3f(-4.5,  3.0, -5.01);
		vrRenderText(buffer);

	} else {
		sprintf(buffer, "no head configured.");
		glRasterPos3f(-4.5,  2.0, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_6sensors >= 1) {
		vrMatrix	tmp_matrix;

		vrMatrixGet6sensorValues(&tmp_matrix, 0);
		sprintf(buffer, "%csensor6[0]: %5.2f %5.2f %5.2f",
			(vrGet6sensorOobValue(0) ? ' ' : '>'),
			VRMAT_ROWCOL(&tmp_matrix, VR_X, VR_W), VRMAT_ROWCOL(&tmp_matrix, VR_Y, VR_W), VRMAT_ROWCOL(&tmp_matrix, VR_Z, VR_W));
		glRasterPos3f(-8.5,  1.0, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_6sensors >= 2) {
		sprintf(buffer, "%csensor6[1]: %5.2f %5.2f %5.2f",
			(inputinfo->sensor6[1]->oob ? ' ' : '>'),
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_X, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_Y, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_Z, VR_W));

		glRasterPos3f(-8.5,  0.5, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_6sensors >= 3) {
		sprintf(buffer, "%csensor6[2]: %5.2f %5.2f %5.2f",
			(inputinfo->sensor6[2]->oob ? ' ' : '>'),
			VRMAT_ROWCOL(inputinfo->sensor6[2]->position, VR_X, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[2]->position, VR_Y, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[2]->position, VR_Z, VR_W));

		glRasterPos3f(-8.5,  0.0, -5.01);
		vrRenderText(buffer);
	}

	glPopAttrib();
  }
#else /* } { */

#endif /* } */

	renderTime = vrCurrentWallTime() - beginRenderTime;

	/* set the text color */
	glColor3ub(155, 155, 155);

	/* just print how long it takes to render */
	sprintf(buffer, "render time: %8.8f", (float)(renderTime));
	glRasterPos3f(-4.5,  8.0, -5.01);
	vrRenderText(buffer);

#endif /* } FREEVR */

#ifndef LIMITED_TRACE
	vrTrace("draw_world", "ending");
#endif

}


/*********************************************************************/
/* draw_valuator_world(): draw a simple world for testing valuators. */
/*********************************************************************/
void draw_valuator_world(vrRenderInfo *rendinfo)
{
	char		buffer[2048];	/* this could be smaller, but is now safer against      */
					/*   memory overflow bugs.  TODO: replace w/ sprintf's  */
					/*   with snprintf when that becomes ubiquitous -- it's */
					/*   not in IRIX 6.2                                    */
	vrInputInfo	*inputinfo;	/* Pointer to the overall input data structure */

	inputinfo = rendinfo->context->input;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/************************************************************/
	/*** a simple two object grey world for red-green testing ***/

	/* a 4' 50% grey cube in front of the cave */
	glPushMatrix();
	glColor3ub(128, 128, 128);
	glTranslatef(-2.5, 0.0, -2.5);
	glScalef(1.0, 1.0, 1.0);
	vrShapeGLCubePlusOutline();
	glPopMatrix();

	glPushMatrix();
#if defined(FREEVR) /* { */
	vrRenderTransformUserTravel();
#endif /* } */

	/* an 80% grey pyramid on the floor of the cave near the front wall */
	glPushMatrix();
	glColor3ub(204, 204, 204);
	glTranslatef(-2.3, 6.2, -0.5);
	glScalef(1.5, 1.5, 1.5);
	glRotatef(90.0, 1.0, 0.0, 0.0);
	vrShapeGLPyramidPlusOutline();
	glPopMatrix();

	glPopMatrix();	/* back to RW coordinate space */


#if defined(FREEVR) /* { */
  {
	int		count;

	/*******************************************************/
	/** Print some boxes representing valuator input info **/
	/*******************************************************/
	for (count = 0; count < inputinfo->num_valuators; count++) {
		glPushMatrix();
		glColor3ub(84, 134, 72);
		glTranslatef(-4.5 + (0.35 * count), 0.0, 0.5);
		glScalef(0.1, 4.0 * vrGetValuatorValue(count), 0.1);
		vrShapeGLCubePlusOutline();
		glPopMatrix();
	}

	/*******************************************************/
	/** Now display boxes from N-sensor[0] (if available) **/
	/*******************************************************/
	if (inputinfo->num_Nsensors > 0) {
		for (count = 0; count < 22; count++) {
			glPushMatrix();
			glColor3ub(84, 72, 134);
			glTranslatef(-4.5 + (0.35 * count), 0.0, 2.5);
			glScalef(0.1, 4.0 * vrGetNsensorValue(0, count), 0.1);
			vrShapeGLCubePlusOutline();
			glPopMatrix();
		}
	}

  }
	/**************************/
	/** Print some text info **/
	/**************************/
  {
#include "vr_math.h"
#include "vr_procs.h"
	vrMatrix	head;
	int		count;
	vrProcessInfo	*input_proc;

	/* first store, then adjust some graphics attributes for text rendering */
	/*      disregard z-buffer -- always display the text on top */
	glPushAttrib(GL_DEPTH_BUFFER_BIT);
	glDisable(GL_DEPTH_TEST);

	/* set the text color */
	glColor3ub(155, 155, 155);


	/* render on front wall */

	sprintf(buffer, "Shared memory usage:%7ld (%ld freed)", vrShmemUsage(), vrShmemFreed());
	glRasterPos3f(-10.5,  12.5, -5.01);
	vrRenderText(buffer);

	input_proc = inputinfo->sensor6[0]->my_device->proc;
	sprintf(buffer, "input info:%9ld %6.2f(1) %6.2f(10)",
		input_proc->frame_count, input_proc->fps1, input_proc->fps10);
	glRasterPos3f(-10.5,  11.5, -5.01);
	vrRenderText(buffer);

	sprintf(buffer, "render info:%8ld %6.2f(1) %6.2f(10)",
		rendinfo->frame_count, vrThisProc->fps1, vrThisProc->fps10);
	glRasterPos3f(-10.5,  10.5, -5.01);
	vrRenderText(buffer);

	sprintf(buffer, "sim time: %8.4f", (double)(rendinfo->frame_stime));
	glRasterPos3f(-4.5,  9.0, -5.01);
	vrRenderText(buffer);

	glRasterPos3f(-4.5,  8.0, -5.01);
	sprintf(buffer, "buttons:");
	for (count = 0; count < inputinfo->num_2ways; count++) 
		sprintf(buffer, "%s %d", buffer, vrGet2switchValue(count));

	vrRenderText(buffer);

	glRasterPos3f(-4.5,  7.0, -5.01);
	sprintf(buffer, "vals:");
	for (count = 0; count < inputinfo->num_valuators; count++)
		sprintf(buffer, "%s%5.2f", buffer, vrGetValuatorValue(count));
	vrRenderText(buffer);

	if (vrMatrixGetRWFromUserHead(&head, 0) != NULL) {
		vrPoint	*leye = &(vrCurrentEye()->loc);

		sprintf(buffer, "head: %5.2f %5.2f %5.2f", VRMAT_ROWCOL(&head, VR_X, VR_W), VRMAT_ROWCOL(&head, VR_Y, VR_W), VRMAT_ROWCOL(&head, VR_Z, VR_W));
		glRasterPos3f(-5.0,  6.0,  4.50);
		vrRenderText(buffer);
		glRasterPos3f(-4.5,  6.0, -5.01);	/* ditto this to left wall */
		vrRenderText(buffer);

		sprintf(buffer, "leye: %5.2f %5.2f %5.2f", leye->v[VR_X], leye->v[VR_Y], leye->v[VR_Z]);
		glRasterPos3f(-4.5,  4.0, -5.01);
		vrRenderText(buffer);

	} else {
		sprintf(buffer, "no head configured.");
		glRasterPos3f(-4.5,  3.0, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_6sensors >= 1) {
		vrMatrix	tmp_matrix;

		vrMatrixGet6sensorValues(&tmp_matrix, 0);
		sprintf(buffer, "sensor6[0]: %5.2f %5.2f %5.2f",
			VRMAT_ROWCOL(&tmp_matrix, VR_X, VR_W), VRMAT_ROWCOL(&tmp_matrix, VR_Y, VR_W), VRMAT_ROWCOL(&tmp_matrix, VR_Z, VR_W));

		glRasterPos3f(-8.5,  2.0, -5.01);
		vrRenderText(buffer);
	}

	if (inputinfo->num_6sensors >= 2) {
		sprintf(buffer, "sensor6[1]: %5.2f %5.2f %5.2f",
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_X, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_Y, VR_W),
			VRMAT_ROWCOL(inputinfo->sensor6[1]->position, VR_Z, VR_W));

		glRasterPos3f(-8.5,  1.0, -5.01);
		vrRenderText(buffer);
	}
	glPopAttrib();
  }
#endif /* } FREEVR */

}

