/* ======================================================================
 * 
 * HH   HH         vr_context.h
 * HH   HH         Author(s): Bill Sherman
 * HHHHHHH         Created: June 21, 2001
 * HH   HH         Last Modified: June 3, 2003
 * HH   HH
 * 
 * Header file for FreeVR context information.  Defines the
 * vrContext info structure (which contains pointers to the other
 * primary info structures -- input & config), which is the
 * single entry point to context configuration.
 * 
 * Copyright 2003, Bill Sherman & Friends, All rights reserved.
 * With the intent to provide an open-source license to be named later.
 * ====================================================================== */
#ifndef __VRCONTEXT_H__
#define __VRCONTEXT_H__

#define FREEVR_VERSION_MAJOR	0
#define FREEVR_VERSION_MINOR	4
#define FREEVR_VERSION_SUB	g
#define FREEVR_VERSION		000407		/* integer encoding for 0.4g */

#include "vr_shmem.h"		/* for vrLock type definition, includes vr_enums.h */

typedef struct vrContextInfo_st {
		/***************************/
		/* Generic Object field(s) */
		vrObject	object_type;	/* The type of this object (ie. VROBJECT_CONTEXT) */
		/* NOTE: the rest of the generic object fields not included with individual inputs*/
		/* TODO: consider adding the other object fields to the input structures.         */

		/***************************/
		char		*version;	/* specifies which version of the library this is */
		char		*arch;		/* specifies the architecture (and abi) of compilation */
		char		*compile;	/* specifies information about the compile-options used for this library */
		char		*homedir;	/* specifies the home directory under which system FreeVR files can be found */

		size_t		shmem_size;	/* Must be declared prior to reading config */
	struct vrShmemInfo_st	*shmem;		/* Information about the shared memory system   TODO: use this. */

		unsigned int	startup_error;	/* Flags for any problems encountered during startup of the library */
		vrSystemStatus	status;		/* The status of the FreeVR system (of this context) */
		unsigned int	paused;		/* Flag indicating system is/should-be paused */

		vrTime		time_immemorial;/* the time when vrStart was called         */
		vrTime		paused_time;	/* the amount of time spent paused          */
		vrTime		pause_wtime;	/* the time at which pausing began          */

	struct vrConfigInfo_st	*config;	/* pointer to the configuration structure   */
	struct vrInputInfo_st	*input;		/* pointer to the inputs structure          */
	struct vrObjectLists_st	*object_lists;	/* pointer to the possible config objects   */
	struct vrCallbackList_st *callbacks;	/* pointer to the system callbacks          */
		vrLock		head_lock;	/* pointer to the head of the list of locks */
		vrLock		tail_lock;	/* pointer to the tail of the list of locks */
	struct vrBarrier_st	*head_barrier;	/* pointer to the head of the list of barriers */
	struct vrBarrier_st	*tail_barrier;	/* pointer to the tail of the list of barriers */
		vrLock		barrier_lock;	/* to prevent simultaneous modification of the list */

		vrLock		print_lock;	/* a lock to keep printf's from overwriting */

	} vrContextInfo;

#if 1 /* if would be preferable if a global scoped variable wasn't necessary. */
	/*  currently required by vr_debug.c and vr_math.c::vrStartTime().  */
extern vrContextInfo *vrContext;
#endif

#endif /* __VRCONTEXT_H__ */
