/* ======================================================================
 * 
 * HH   HH         vr_entity.h
 * HH   HH         Author(s): Bill Sherman
 * HHHHHHH         Created: December 21, 1998
 * HH   HH         Last Modified: April 3, 2003
 * HH   HH
 * 
 * Header file for FreeVR entity structures (ie. Eyes -- this would
 * normally also include Users & Props, but I tried to make a vr_entity.h
 * file a couple months ago and move that stuff from vr_input.h, but I
 * recall that didn't go very well and I had to move everything back).
 * So, right now Eyes and EyeLists are declared.
 * 
 * Copyright 2003, Bill Sherman & Friends, All rights reserved.
 * With the intent to provide an open-source license to be named later.
 * ====================================================================== */
#ifndef __VRENTITY_H__
#define __VRENTITY_H__

#include "vr_math.h"
#include "vr_context.h"		/* which includes vr_shmem.h and vr_enums.h */

#ifdef __cplusplus
extern "C" {
#endif


/******************************************************************************/
/* NOTE: type definitions for vrUserInfo and vrPropInfo (which are entities)  */
/*   are NOT found in this file, but rather in the vr_input.h header file.    */
/*   This was done to simplify which headers must include which other headers.*/
/******************************************************************************/

typedef struct vrEyeInfo_st {
		/***************************/
		/* Generic Object field(s) */
		vrObject	object_type;	/* The type of this object (ie. VROBJECT_EYEINFO) */
		/* NOTE: the rest of the generic object fields not included with individual inputs*/
		/* TODO: consider adding the other object fields to the input structures.         */

		/***************************/
		vrEyeType	type;
		int		num;		/* the number of this eye in running system (1-based) */
	struct	vrUser_st	*user;
		vrAnaglyphType	color;
		vrFrameBufferType render_frame;	/* the frame buffer to render in */
		char		*name;		/* CONFIG: this is the string parsed into the other fields */
	struct	vrEyeInfo_st	*next;		/* the next eye in a linked list */

		vrPoint		loc;		/* location of the eye */
	} vrEyeInfo;


typedef struct vrEyelistInfo_st {

		/* Generic Object fields */
		vrObject	object_type;	/* The type of this object (ie. VROBJECT_EYELIST) */
		int		id;		/* numeric id for this eyelist. */
		char		*name;		/* CONFIG: The name assigned to this eyelist. */
		int		malleable;	/* CONFIG: Whether the data of this eyelist can be modified */
	struct vrEyelistInfo_st	*next;		/* the next eyelist in a linked list */
		vrContextInfo	*context;	/* pointer to the vrContext memory structure */
		int		line_created;	/* line number of where this object was created in config */
		int		line_lastmod;	/* line number of last time this object modified in config */
		char		file_created[512];/* file (or other) where this object was created*/
		char		file_lastmod[512];/* file (or other) where this object was created*/

		/* Fields specific to EyeLists */
		char		**monofb_names;	/* list of eye names -- to be parsed */
		int		num_monofb_names;/* number in list of names */
		vrEyeInfo	*monofb;	/* or this could be "single" */
		char		**leftfb_names;	/* list of eye names -- to be parsed */
		int		num_leftfb_names;/* number in list of names */
		vrEyeInfo	*leftfb;	/* or this could be "dualfb_left" */
		char		**rightfb_names;/* list of eye names -- to be parsed */
		int		num_rightfb_names;/* number in list of names */
		vrEyeInfo	*rightfb;	/* or this could be "dualfb_right" */
		char		**leftvp_names;	/* list of eye names -- to be parsed */
		int		num_leftvp_names;/* number in list of names */
		vrEyeInfo	*leftvp;	/* or this could be "dualvp_left" */
		char		**rightvp_names;/* list of eye names -- to be parsed */
		int		num_rightvp_names;/* number in list of names */
		vrEyeInfo	*rightvp;	/* or this could be "dualvp_right" */
		char		**anaglfb_names;/* list of eye names -- to be parsed */
		int		num_anaglfb_names;/* number in list of names */
		vrEyeInfo	*anaglfb;	/* or this could be "single" */

	} vrEyelistInfo;


/***** Functions *****/

char		*vrEyeTypeName(vrEyeType type);
vrEyeType	vrEyeValue(char *name);
char		*vrAnaglyphTypeName(vrAnaglyphType type);
vrAnaglyphType	vrAnaglyphValue(char *name);
char		*vrPrintStyleName(vrPrintStyle style);
vrPrintStyle	vrPrintStyleValue(char *name);
vrEyeInfo	*vrMakeEye(vrContextInfo *context, char *eyestring);
void		vrFprintEyeInfo(FILE *file, vrEyeInfo *eyeinfo, vrPrintStyle style);
void		vrEyelistInitialize(vrContextInfo *context, vrEyelistInfo *eyelistinfo);
#if 0 /* vrUserInfo  & vrPropInfo require vr_input.h which causes a circular include */
	/* TODO: fix this by using "struct" -- 3/3/3: didn't work */
void		vrUserClear(struct vrUserInfo_st *object);
void		vrUserCopy(struct vrUserInfo_st *dest_object, struct vrUserInfo_st *src_object);
void		vrFprintUserInfo(FILE *file, struct vrUserInfo_st *userinfo, vrPrintStyle style);
void		vrPropClear(struct vrPropInfo_st *object);
void		vrPropCopy(struct vrPropInfo_st *dest_object, struct vrPropInfo_st *src_object);
void		vrPropFreezeVisren(vrContextInfo *context);
void		vrFprintPropInfo(FILE *file, struct vrPropInfo_st *propinfo, vrPrintStyle style);
#endif
void		vrUserInitialize(vrContextInfo *context);
int		vrUsersInitialized(vrContextInfo *context);
vrMatrix	*vrMatrixGetRWFromUserHead(vrMatrix *headmat, int usernum);
vrPoint		*vrPointGetVWUserFromRWPoint(vrPoint *virtual_pnt, int usernum, vrPoint *real_pnt);
vrPoint		*vrPointGetRWFromVWUserPoint(vrPoint *real_pnt, int usernum, vrPoint *virtual_pnt);
vrPointf	*vrPointfGetRWFromVWUserPointf(vrPointf *real_pnt, int usernum, vrPointf *virtual_pnt);
vrVector	*vrVectorUserVWFromRW(int usernum, vrVector *virtual_vec, vrVector *real_vec);
vrPoint		*vrPointGetRWLocationFromMatrix(vrPoint *real_point, vrMatrix *mat);
vrPoint		*vrPointGetVWFromUserMatrix(vrPoint *virtual_point, int usernum, vrMatrix *mat);

void		vrRenderTransformWindow(void);
void		vrRenderTransformUserTravel(void);
vrMatrix	*vrMatrixGetUserTravel(vrMatrix *mat, int usernum);
vrMatrix	*vrMatrixGetUserTravelInv(vrMatrix *mat, int usernum);
int		vrUserTravelReset(int usernum);
int		vrUserTravelTranslate3d(int usernum, double x, double y, double z);
int		vrUserTravelTranslateAd(int usernum, double *array);
int		vrUserTravelTranslateVec(int usernum, vrVector *vec);
int		vrUserTravelRotateId(int usernum, int axis, double theta);
int		vrUserTravelScale(int usernum, double scale);
int		vrUserTravelTransformMatrix(int usernum, vrMatrix *mat);
void		vrUserTravelFreezeVisren(vrContextInfo *context);

void		vrRenderTransform6sensorDirect(/* vr6sensor *sensor6 -- can't include vr_input.h here */);
void		vrRenderTransform6sensor(int sensor6_num);

void		vrEyelistClear(vrEyelistInfo *object);
void		vrEyelistCopy(vrEyelistInfo *dest_object, vrEyelistInfo *src_object);
void		vrFprintEyelistInfo(FILE *file, vrEyelistInfo *eyelistinfo, vrPrintStyle style);


#ifdef __cplusplus
}
#endif

#endif  /* __VRENTITY_H__ */
