/* ======================================================================
 * 
 * HH   HH         vr_input.opts.h
 * HH   HH         Author(s): Bill Sherman
 * HHHHHHH         Created: August 17, 1998
 * HH   HH         Last Modified: January 21, 2003
 * HH   HH
 * 
 * Header file for available FreeVR input device options
 * 
 * Copyright 2003, Bill Sherman & Friends, All rights reserved.
 * With the intent to provide an open-source license to be named later.
 * ====================================================================== */
/**************************************************************************/
/* Library programmers: add new input device options here.                */
/*                                                                        */
/* To add a new type "T", add the declaration for the initialization      */
/*   function and add a new record to the vrInputOpts array.  The new     */
/*   record can be placed anywhere before the all NULL record.  The first */
/*   field of the record is a string representation of the new type,      */
/*   followed by a single function for putting the specific device info   */
/*   into the input device structure.                                     */
/*                                                                        */
/* The function can be named anything, but must take a specific argument: */
/*   function1(vrInputDevice *)                                           */
/* The function must create all the necessary callbacks for opening/      */
/*   polling/resetting/closing the device, and return "void".             */
/*                                                                        */
/* Also, don't forget to add the new source file in the Makefile.         */
/**************************************************************************/

#include "vr_input.h"

void vrStaticInitInfo(vrInputDevice *info);
void vrShmemdInitInfo(vrInputDevice *info);
void vrFobInitInfo(vrInputDevice *info);
void vrJoydevInitInfo(vrInputDevice *info);
void vrMagellanInitInfo(vrInputDevice *info);
void vrSpacetecInitInfo(vrInputDevice *info);
void vrPinchgloveInitInfo(vrInputDevice *info);
void vrTrackdInitInfo(vrInputDevice *info);

#ifdef WIN_GLX
void vrXwindowsInitInfo(vrInputDevice *info);
#endif

#ifdef WIN_WGL
void vrWin32InitInfo(vrInputDevice *info);
#endif

#ifdef VRINPUT_ONLY /* { */
vrInputOptsType vrInputOpts[] = {
		{ "static",	vrStaticInitInfo },
		{ "shmemd",	vrShmemdInitInfo },
#ifdef WIN_GLX
		{ "xwindows",	vrXwindowsInitInfo },
#endif
#ifdef WIN_WGL
		{ "win32",      vrWin32InitInfo },
#endif
		{ "asc_fob",	vrFobInitInfo },
		{ "joydev",	vrJoydevInitInfo },
		{ "magellan",	vrMagellanInitInfo },
		{ "spaceorb",	vrSpacetecInitInfo },
		{ "spaceball",	vrSpacetecInitInfo },
		{ "pinchglove",	vrPinchgloveInitInfo },
#if 0
		{ "trackd",	vrTrackdInitInfo },	/* not globally available */
#endif
		{ NULL, NULL }
	};
#endif /* } VRINPUT_ONLY */

