/* ======================================================================
 * 
 *  CCCCC          vr_utils.c
 * CC   CC         Author(s): John Stone, Bill Sherman
 * CC              Created: August 10, 2000
 * CC   CC         Last Modified: June 4, 2003
 *  CCCCC
 * 
 * Code file for miscellaneous machine and operating system dependent 
 * utility routines.
 * 
 * Copyright 2003, Bill Sherman & Friends, All rights reserved.
 * With the intent to provide an open-source license to be named later.
 * ====================================================================== */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "vr_utils.h"
#include "vr_debug.h"

/************************************************************/
void vrSleep(long useconds) {

	if (useconds <= 0) {
		return;
	}

#if defined(SGINAP)
	/* only use sginap() on IRIX 5.x where usleep is not available */
	sginap(useconds / 10000);
#else
	/* handles delays longer than one second */
	while (useconds >= 1000000) {
		usleep(999999);
		useconds -= 999999;
	}
#  if !defined(__linux) /* on systems that produce higher-res waits with calls less than 10000ms, this can improve res */
	while (useconds >= 5000) {
		usleep(5000);
		useconds -= 5000;
	}
#  endif
	usleep(useconds);
#endif
}


/**********************************************************************/
void vrShellCmd(char *cmdstr, char *name, int pid, int error_code)
{
	if (cmdstr == NULL)
		return;

	if (strlen(cmdstr) > 1) {
		int	no_subst;		/* true when no subsitution done thus far in a loop */
		char	syscmd[1024];
		char	syscmd2[1024];
		char	*subst = NULL;
		char	*percent_loc = NULL;

		strcpy(syscmd, cmdstr);

		/* loop until no substitutions were done */
		/*   NOTE: we do this to make sure multiple        */
		/*   instances of the same substitution will work. */
		do {
			no_subst = 1;

			/* replace "#P" with the process pid */
			subst = strstr(syscmd, "#P");
			if (subst != NULL) {
				/* replace all occurances of '%' w/ "%%" first */
				while ((percent_loc = strchr(syscmd, '%')) != NULL) {
					strncpy(syscmd2, syscmd, (size_t)(percent_loc - syscmd));
					strcat(syscmd2, percent_loc);
				}

				/* now do the replacement */
				strncpy(syscmd2, syscmd, (subst - syscmd));
				syscmd2[subst - syscmd] = '\0';
				strcat(syscmd2, "%d");
				strcat(syscmd2, &(subst[2]));
				sprintf(syscmd, syscmd2, pid); /* TODO: replace with snprintf, which doesn't exist in IRIX 6.2 */

				no_subst = 0;
			}

			/* replace "#N" with the process name */
			subst = strstr(syscmd, "#N");
			if (subst != NULL) {
				/* replace all occurances of '%' w/ "%%" first */
				while ((percent_loc = strchr(syscmd, '%')) != NULL) {
					strncpy(syscmd2, syscmd, (size_t)(percent_loc - syscmd));
					strcat(syscmd2, percent_loc);
				}

				/* now do the replacement */
				strncpy(syscmd2, syscmd, (subst - syscmd));
				syscmd2[subst - syscmd] = '\0';
				strcat(syscmd2, "%s");
				strcat(syscmd2, &(subst[2]));
				sprintf(syscmd, syscmd2, name); /* TODO: replace with snprintf, which doesn't exist in IRIX 6.2 */

				no_subst = 0;
			}

			/* replace "#E" with the error code */
			subst = strstr(syscmd, "#E");
			if (subst != NULL) {
				/* replace all occurances of '%' w/ "%%" first */
				while ((percent_loc = strchr(syscmd, '%')) != NULL) {
					strncpy(syscmd2, syscmd, (size_t)(percent_loc - syscmd));
					strcat(syscmd2, percent_loc);
				}

				/* now do the replacement */
				strncpy(syscmd2, syscmd, (subst - syscmd));
				syscmd2[subst - syscmd] = '\0';
				strcat(syscmd2, "%d");
				strcat(syscmd2, &(subst[2]));
				sprintf(syscmd, syscmd2, error_code); /* TODO: replace with snprintf, which doesn't exist in IRIX 6.2 */

				no_subst = 0;
			}

		} while (no_subst == 0);

		system(syscmd);
	}
}


/************************************************************/
/* Okay, this was intended to search a list of arbitrary length */
/*   strings, but of course that was a rediculus expectation of */
/*   C.  So for now, don't use this function.  There must be a  */
/*   reasonable solution, but for the moment we'll have a version */
/*   that requires 256 length strings.                          */
int vrSearchStringList(char *item, int list_length, char *list[])
{
	int	count;

	vrDbgPrintfN(1, "vrSearchStringList(%s, %d, 0x%p)\n", item, list_length, list);
	for (count = 0; count < list_length; count++) {
		vrDbgPrintfN(1, "...vrSearchStringList() %d: comparing item with '%s'(0x%p)\n", count, list[count], list[count]);
		if (strcmp(item, list[count]) == 0) {
			vrDbgPrintfN(1, "...vrSearchStringList() returning %d\n", count);
			return count;
		}
	}

	vrDbgPrintfN(1, "...vrSearchStringList() returning -1\n");
	return -1;
}


/************************************************************/
/* a version of the above that assumes (ie. requires) that the */
/*   strings all have a length of 256 bytes.                   */
int vrSearch256StringList(char *item, int list_length, char list[][256])
{
	int	count;

	vrDbgPrintfN(1, "vrSearch256StringList(%s, %d, 0x%p)\n", item, list_length, list);
	for (count = 0; count < list_length; count++) {
		vrDbgPrintfN(1, "...vrSearchStringList() %d: comparing item with '%s'(0x%p)\n", count, list[count], list[count]);
		if (strcmp(item, list[count]) == 0) {
			vrDbgPrintfN(1, "...vrSearch256StringList() returning %d\n", count);
			return count;
		}
	}

	vrDbgPrintfN(1, "...vrSearch256StringList() returning -1\n");
	return -1;
}

